<?php

namespace Shopping\Product\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Shopping\Product\Events\ProductWorkflow as ProductWorkflowEvent;
use Shopping\Product\Notifications\ProductWorkflow as ProductWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class ProductWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(ProductWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $product is submitted.
     *
     * @param ProductWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(ProductWorkflowEvent $event)
    {
        $client = $event->product->client;
        Notification::send($client, new ProductWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $product is published.
     *
     * @param ProductWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(ProductWorkflowEvent $event)
    {
        $client = $event->product->client;
        Notification::send($client, new ProductWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $product is approved.
     *
     * @param ProductWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(ProductWorkflowEvent $event)
    {
        $client = $event->product->client;
        Notification::send($client, new ProductWorkflowNotification($event));
    }

    /**
     * Handles the $product workflow event as a listener.
     *
     * @param ProductWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(ProductWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
